# -*- encoding: utf-8 -*-
#
#   slovima.py - Write number as text in Serbian Calc AddIn
#
#   Copyright (C) 2008 Goran Rakic <grakic@devbase.net>
#   Based upon http://www.praktikum.co.yu/office/excel/xl_013.asp
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#   02110-1301, USA.
#

import uno
import unohelper

from com.sun.star.lang import IllegalArgumentException, Locale

# pythonloader looks for a static g_ImplementationHelper variable
g_ImplementationHelper = unohelper.ImplementationHelper()

# used for storing supported interfaces of classes
g_TypeTable = {}

class SlovimaAddin:
    def __init__(self, ctx):
        self.ctx = ctx  # store the default contextt for later use
        self.ServiceName = "org.openoffice.sheet.addin.SlovimaAddin"
        self.AddInService = "com.sun.star.sheet.AddIn"
        self.ImplementationName = "org.openoffice.sheet.addin.SlovimaAddin"
        self.SupportedServiceNames = (self.ServiceName, self.AddInService,)
        self.Locale = -1

    # XTypeProvider method implementations
    def getTypes (self):
        global g_TypeTable

        if g_TypeTable.has_key (self.__class__):
            ret = g_TypeTable[self.__class__]
        else:
            ret = (
                uno.getTypeByName( "org.openoffice.sheet.addin.XSlovimaAddin" ),
                uno.getTypeByName( "com.sun.star.sheet.XAddIn" ),
                uno.getTypeByName( "com.sun.star.lang.XLocalizable" ),
                uno.getTypeByName( "com.sun.star.lang.XServiceName" ),
                uno.getTypeByName( "com.sun.star.lang.XServiceInfo" ),
                uno.getTypeByName( "com.sun.star.lang.XTypeProvider" ),
                )
            g_TypeTable[self.__class__] = ret
        return ret

    def getImplementationId (self):
        return uno.generateUuid ()

    # XServiceName method implementations
    def getServiceName(self):
        return self.ServiceName

    # XServiceInfo method implementations
    def getImplementationName (self):
        return self.ImplementationName

    def supportsService(self, ServiceName):
        return (ServiceName in self.SupportedServiceNames)

    def getSupportedServiceNames (self):
        return self.SupportedServiceNames

    # XLocalizable method implementations
    def setLocale(self, eLocale):
        # the locale is stored and used for getLocale, but otherwise
        # ignored at the moment
        self.Locale = eLocale

    def getLocale(self):
        return self.Locale

    # -----------------------------------------------------------------------
    # XSlovimaAddin method implementations ====================================

    def slovima(self, nValue):
        return self.slovimaImpl(nValue, 0)


    def slovimal(self, nValue):
        return self.slovimaImpl(nValue, 1)


    # XAddIn method implementations
    def getProgrammaticFuntionName(self, aDisplayName):
        # Attention: The method name contains a spelling error.
        # Due to compatibility reasons the name cannot be changed.
        # it maps between the internal function name, as defined in the interface and the
        # function name as shown to the user of the spreadsheet application
        if aDisplayName == 'slovima':
            return "SLOVIMA"
        if aDisplayName == 'slovimal':
            return "SLOVIMAL"
        else:
            return ""

    def getDisplayFunctionName(self, aProgrammaticName):
        # maps between internal name and function name
        return aProgrammaticName

    def getFunctionDescription(self, aProgrammaticName):
        # Description for users
        if aProgrammaticName == 'slovima':
            return u"враћа број исписан словима, ћирилицом"
        if aProgrammaticName == 'slovimal':
            return u"враћа број исписан словима, латиницом"
        else:
            return u"није дефинисано"

    def getDisplayArgumentName(self, aProgrammaticFunctionName, nArgument):
        # Name of parameters for users
        if aProgrammaticFunctionName == 'slovima' or aProgrammaticFunctionName == 'slovimal':
            return [u"број"][nArgument]
        else:
            return u"није дефинисано"

    def getArgumentDescription(self, aProgrammaticFunctionName, nArgument):
        # Description of parameters for users
        if aProgrammaticFunctionName == 'slovima' or  aProgrammaticFunctionName == 'slovimal':
            return [u"вредност"][nArgument]
        else:
            return u"није дефинисано"

    def getProgrammaticCategoryName(self, aProgrammaticFunctionName):
        # sorts each add- in functions into one of the spread sheet
        # application's function categories.
        return "Text"

    def getDisplayCategoryName(self, aProgrammaticFunctionName):
        return "Text"


    # -----------------------------------------------------------------------
    # AddIn method implementations ====================================

    _s = {
        0:[u"нула", u"nula"],
        1:[u"један", u"jedan"],
        2:[u"два", u"dva"],
        3:[u"три", u"tri"],
        4:[u"четири", u"četiri"],
        5:[u"пет", u"pet"],
        6:[u"шест", u"šest"],
        7:[u"седам", u"sedam"],
        8:[u"осам", u"osam"],
        9:[u"девет", u"devet"],
        'стотину':[u"стотину", u"stotinu"],
        'стотине':[u"стотине", u"stotine"],
        'стотина':[u"стотина", u"stotina"],
        'четр':[u"четр", u"četr"],
        'шез':[u"шез", u"šez"],
        'пе':[u"пе", u"pe"],
        'деве':[u"деве", u"deve"],
        'десет':[u"десет", u"deset"],
        'једа':[u"једа", u"jeda"],
        'четр':[u"четр", u"četr"],
        'наест':[u"наест", u"naest"],
        'једна':[u"једна", u"jedna"],
        'две':[u"две", u"dve"],
        'хиљад':[u"хиљад", u"hiljad"],
        'милион':[u"милион", u"milion"],
        'милијард':[u"милијард", u"milijard"],
        'билион':[u"билион", u"bilion"],
        'трилион':[u"трилион", u"trilion"],
        'а':[u"а", u"a"],
        'у':[u"у", u"u"],
        'е':[u"е", u"e"],
        'и':[u"и", u"i"]
    }
    
    def slovimaImpl(self, nValue, cyrlat):

        if nValue == 0:
            return SlovimaAddin._s[0][cyrlat]

        celi = int(nValue)
        stepen = 0
        rez = ""
        
        while celi > 0:
            
            rezt = ""
            
            # uzmi tri naredne cifre sa kraja
            cs = celi % 1000
            cd = cs % 100
            cj = cd % 10

            cs = int(cs/100)
            cd = int(cd/10)

            print celi, cs, cd, cj
            
            if cs or cd or cj:

                if cs == 2:
                    rezt += SlovimaAddin._s["две"][cyrlat]
                elif cs > 2:
                    rezt += SlovimaAddin._s[cs][cyrlat]
                
                if cs == 1:
                    rezt += SlovimaAddin._s["стотину"][cyrlat]
                elif cs > 1 and cs < 5:
                    rezt += SlovimaAddin._s["стотине"][cyrlat]
                elif cs >= 5:
                    rezt += SlovimaAddin._s["стотина"][cyrlat]
                
                rezj = ""
                if cj != 0:
                    rezj = SlovimaAddin._s[cj][cyrlat]
                
                if cd == 4:
                    rezt += SlovimaAddin._s["четр"][cyrlat]
                elif cd == 6:
                    rezt += SlovimaAddin._s["шез"][cyrlat]
                elif cd == 5:
                    rezt += SlovimaAddin._s["пе"][cyrlat]
                elif cd == 9:
                    rezt += SlovimaAddin._s["деве"][cyrlat]
                elif cd in [2, 3, 7, 8]:
                    rezt += SlovimaAddin._s[cd][cyrlat]
                elif cd == 1:
                    rezj = ""
                    if cj == 0:
                        rezt += SlovimaAddin._s["десет"][cyrlat]
                    elif cj == 1:
                        rezt += SlovimaAddin._s["једа"][cyrlat]
                    elif cj == 4:
                        rezt += SlovimaAddin._s["четр"][cyrlat]
                    else:
                        rezt += SlovimaAddin._s[cj][cyrlat]
                    if cj > 0:
                        rezt += SlovimaAddin._s["наест"][cyrlat]
                        
                if cd > 1:
                    rezt += SlovimaAddin._s["десет"][cyrlat]

                if (stepen == 1 or stepen == 3) and cd != 1:
                    if cj == 1:
                        rezj = SlovimaAddin._s["једна"][cyrlat]
                    elif cj == 2:
                        rezj = SlovimaAddin._s["две"][cyrlat]
                
                rezt += rezj

                tmp = cd*10 + cj
                if stepen == 1:
                    rezt += SlovimaAddin._s["хиљад"][cyrlat]
                    if (tmp > 11 and tmp < 19) or cj == 1:
                        rezt += SlovimaAddin._s["а"][cyrlat]
                    elif cs == 0 and cd == 0 and cj == 1:
                        rezt += SlovimaAddin._s["у"][cyrlat]
                    elif cj > 4 or cj == 0:
                        rezt += SlovimaAddin._s["а"][cyrlat]
                    elif cj > 1:
                        rezt += SlovimaAddin._s["е"][cyrlat]
                if stepen == 2:
                    rezt += SlovimaAddin._s["милион"][cyrlat] 
                    if (tmp >= 11 and tmp < 19) or cj != 1:
                        rezt += SlovimaAddin._s["а"][cyrlat]                                    
                if stepen == 3:
                    rezt += SlovimaAddin._s["милијард"][cyrlat] 
                    if tmp >= 11 and tmp < 19:
                        rezt += SlovimaAddin._s["и"][cyrlat] 
                    elif cj == 1:
                        rezt += SlovimaAddin._s["а"][cyrlat] 
                    elif cj > 4 or cj == 0:
                        rezt += SlovimaAddin._s["и"][cyrlat]              
                    elif cj > 1:
                        rezt += SlovimaAddin._s["е"][cyrlat]   
                if stepen == 4:
                    rezt += SlovimaAddin._s["билион"][cyrlat]   
                    if cj > 1 or cd == 1:
                        rezt += SlovimaAddin._s["а"][cyrlat]
                if stepen == 5:
                    rezt += SlovimaAddin._s["трилион"][cyrlat]
                    if cj > 1 or cd == 1:
                        rezt += SlovimaAddin._s["а"][cyrlat]
                    stepen = 0

            rez = rezt + rez
            stepen += 1        
            celi = celi / 1000

        return rez


g_ImplementationHelper.addImplementation(
    SlovimaAddin,
    "org.openoffice.sheet.addin.SlovimaAddin",
    ("org.openoffice.sheet.addin.SlovimaAddin", "com.sun.star.sheet.AddIn"),)



